/********************************************************************************
 * Copyright (C) 2014 by Lorenzo Porta (Vindex17) <vindex17@outlook.it>         *
 *                                                                              *
 * This program is free software; you can redistribute it and/or                *
 * modify it under the terms of the GNU General Public License as               *
 * published by the Free Software Foundation; either version 2 of               *
 * the License or (at your option) version 3 or any later version               *
 * accepted by the membership of KDE e.V. (or its successor approved            *
 * by the membership of KDE e.V.), which shall act as a proxy                   *
 * defined in Section 14 of version 3 of the license.                           *
 *                                                                              *
 * This program is distributed in the hope that it will be useful,              *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of               *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                *
 * GNU General Public License for more details.                                 *
 *                                                                              *
 * You should have received a copy of the GNU General Public License            *
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.        *
 *                                                                              *
 ********************************************************************************/

#include "detailedprogressdialog.h"
#include "detailedprogressdialog.moc"

#include <KGlobalSettings>

DetailedProgressDialog::DetailedProgressDialog(const quint64& cs, const quint64& ms, QWidget* parent, Qt::WindowFlags flags)
    : KDialog(parent, flags), completed_jobs(cs), total_jobs(ms)
{
    QWidget* main_widget = mainWidget();
    setMainWidget(main_widget);
    detailedprogressdialog_ui.setupUi(main_widget);
    setButtons(KDialog::Cancel);
    resize((KGlobalSettings::desktopGeometry(main_widget).width() / 2), sizeHint().height());
    setModal(true);
    show();
}


void DetailedProgressDialog::update_progress_bars(KJob*, ulong percent)
{
    detailedprogressdialog_ui.current_job_progressBar->setValue(percent);
    detailedprogressdialog_ui.overall_workload_progressBar->setValue((completed_jobs * 100 + percent) / total_jobs);
}


void DetailedProgressDialog::update_job_description(KJob*, QString title , QPair< QString, QString > field1, QPair< QString, QString > field2)
{
    job_title_text = title;
    detailedprogressdialog_ui.title_label->setText(title);
    detailedprogressdialog_ui.field1_label->setText(i18nc("@info:progress", "%1: %2", field1.first, field1.second));
    detailedprogressdialog_ui.field2_label->setText(i18nc("@info:progress", "%1: %2", field2.first, field2.second));
}


void DetailedProgressDialog::update_job_speed(KJob*, ulong bytepersec)
{
    detailedprogressdialog_ui.title_label->setText(i18nc("@info:progress", "%1 ( @ %2/s )", job_title_text, KGlobal::locale()->formatByteSize(bytepersec)));
}
