/********************************************************************************
 * Copyright (C) 2014 by Lorenzo Porta (Vindex17) <vindex17@outlook.it>         *
 *                                                                              *
 * This program is free software; you can redistribute it and/or                *
 * modify it under the terms of the GNU General Public License as               *
 * published by the Free Software Foundation; either version 2 of               *
 * the License or (at your option) version 3 or any later version               *
 * accepted by the membership of KDE e.V. (or its successor approved            *
 * by the membership of KDE e.V.), which shall act as a proxy                   *
 * defined in Section 14 of version 3 of the license.                           *
 *                                                                              *
 * This program is distributed in the hope that it will be useful,              *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of               *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                *
 * GNU General Public License for more details.                                 *
 *                                                                              *
 * You should have received a copy of the GNU General Public License            *
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.        *
 *                                                                              *
 ********************************************************************************/

#ifndef KFOLDERSYNC_H
#define KFOLDERSYNC_H

#include "detailedprogressdialog.h"
#include "logger.h"
#include "requesterwidget.h"
#include "ui_kfoldersync.h"

#include <kio/directorysizejob.h>
#include <KIO/Job>
#include <KIO/NetAccess>
#include <KProgressDialog>
#include <KStatusNotifierItem>
#include <KXmlGuiWindow>
#include <QDirIterator>
#include <QPointer>
#include <QTimer>

class KFolderSync : public KXmlGuiWindow
{
    Q_OBJECT
    Q_DISABLE_COPY(KFolderSync)

public:
    explicit KFolderSync();
    virtual ~KFolderSync();

private:
    Ui::kfoldersync kfoldersync_ui;
    KStatusNotifierItem tray_obj;
    enum operation_type_enum {sync, bilateral_sync, diff_backup, incr_backup, bilateral_incr_backup, full_backup} operation_type = sync;
    QList<RequesterWidget*> req_widget_list;
    QPointer<KProgressDialog> default_progress_dialog;
    QTimer background_timer;
    int msec_between_attempts;
    bool comparison_made = false;
    bool job_error_found;
    bool operation_must_be_canceled;
    bool log_requested = false;
    bool use_trash = true;
    bool auto_compare = true;
    bool run_in_the_background = false;
    Logger* logger;
    KIO::UDSEntryList current_udsentry_list;
    KIO::filesize_t dir_size_from_job;
    quint64 total_jobs, completed_jobs;
    quint64 f_cp_n;
    quint64 f_ovwr_n;
    quint64 f_old_n;
    quint64 f_del_n;
    quint64 d_cp_n;
    quint64 d_del_n;
    KIO::filesize_t add_sz_n;
    KIO::filesize_t del_sz_n;
    qint64 net_ch_sz_n;

    void setup_actions();
    void update_local_info();
    bool handle_kiojob(KIO::Job* const, const QPointer<DetailedProgressDialog>&);
    void do_sync_or_diff_backup(const KUrl&, const KUrl&, const QPointer<DetailedProgressDialog>&);
    void do_bilateral_sync(const KUrl&, const KUrl&, const QPointer<DetailedProgressDialog>&);
    void do_incr_backup(const KUrl&, const KUrl&, const QPointer<DetailedProgressDialog>&, const KUrl&);
    void do_bilateral_incr_backup(const KUrl&, const KUrl&, const QPointer<DetailedProgressDialog>&);
    void do_full_backup(const KUrl&, KUrl, const QPointer<DetailedProgressDialog>&);
    void analyze_udsentrylist_for_comparison(const KUrl&, const KUrl&);
    void do_operation();

private slots:
    void set_sync_enum(const bool);
    void set_bilateral_sync_enum(const bool);
    void set_diff_backup_enum(const bool);
    void set_incr_backup_enum(const bool);
    void set_bilateral_incr_backup_enum(const bool);
    void set_full_backup_enum(const bool);
    void init_logger(const bool);
    void set_use_trash(const bool);
    void set_auto_compare(const bool);
    void set_run_in_the_background(const bool);
    void update_kdialog_progressBar(KJob*, ulong);
    void check_for_errors(KJob*);
    void run();
    void set_operation_must_be_canceled();
    void trigger_background_operation();
    void come_back_into_the_foreground();
    void save_udsentrylist(KIO::Job*, KIO::UDSEntryList);
    void save_dir_sz(KJob*);
    void notify_comparison_necessity();
    void compare();
    void add_req_widget();
    void del_req_widget(const int);
    void reset_application();

signals:
    void kill_jobs();
};

#endif // _KFOLDERSYNC_H_
