/********************************************************************************
 * Copyright (C) 2014 by Lorenzo Porta (Vindex17) <vindex17@outlook.it>         *
 *                                                                              *
 * This program is free software; you can redistribute it and/or                *
 * modify it under the terms of the GNU General Public License as               *
 * published by the Free Software Foundation; either version 2 of               *
 * the License or (at your option) version 3 or any later version               *
 * accepted by the membership of KDE e.V. (or its successor approved            *
 * by the membership of KDE e.V.), which shall act as a proxy                   *
 * defined in Section 14 of version 3 of the license.                           *
 *                                                                              *
 * This program is distributed in the hope that it will be useful,              *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of               *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                *
 * GNU General Public License for more details.                                 *
 *                                                                              *
 * You should have received a copy of the GNU General Public License            *
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.        *
 *                                                                              *
 ********************************************************************************/

#include "logger.h"
#include "logger.moc"

#include <KAboutData>
#include <KComponentData>
#include <KDateTime>
#include <KLocale>
#include <KMessageBox>
#include <KUrl>
#include <QDir>

Logger::Logger(QWidget* parent, bool& succeeded): QObject(parent), log_file(this)
{
    KUrl log_file_kurl(QDir::homePath());
    log_file_kurl.addPath(i18nc("@item", "%1.log", KGlobal::mainComponent().aboutData()->programName()));
    log_file.setFileName(log_file_kurl.path());
    succeeded = log_file.open(QIODevice::WriteOnly | QIODevice::Append | QIODevice::Text);
    if (succeeded)
        log_text_stream.setDevice(&log_file);
    else
        KMessageBox::sorry(parent, i18nc("@info", "<warning>I can't open nor create <filename>%1</filename>...</warning>", log_file.fileName()));
}

Logger::~Logger()
{
    if (QFileInfo(log_file).size() == 0)
        log_file.remove();
}


void Logger::log_msg(const QString& msg)
{
    log_text_stream << i18nc("@item", "%1 > %2", KDateTime::currentLocalTime().toString(), msg) << endl;
}


void Logger::log_start(const QString& operation, const KUrl& kurl_src, const KUrl& kurl_dest)
{
    log_text_stream << i18nc("@item",
                             "%1 > Starting the %2... [source: <filename>%3</filename> | destination: <filename>%4</filename>]",
                             KDateTime::currentLocalTime().toString(),
                             operation,
                             kurl_src.pathOrUrl(),
                             kurl_dest.pathOrUrl())
                    << endl;
}


void Logger::log_cp(const KUrl& kurl_src, const KUrl& kurl_dest)
{
    log_text_stream << i18nc("@item", "%1 > Copying <filename>%2</filename> to <filename>%3</filename>",
                             KDateTime::currentLocalTime().toString(), kurl_src.pathOrUrl(), kurl_dest.pathOrUrl())
                    << endl;
}


void Logger::log_mv(const KUrl& kurl_src, const KUrl& kurl_dest)
{
    log_text_stream << i18nc("@item", "%1 > Moving old <filename>%2</filename> to <filename>%3</filename>",
                             KDateTime::currentLocalTime().toString(), kurl_src.pathOrUrl(), kurl_dest.pathOrUrl())
                    << endl;
}


void Logger::log_ovwr(const KUrl& kurl)
{
    log_text_stream << i18nc("@item", "%1 > Overwriting <filename>%2</filename>", KDateTime::currentLocalTime().toString(), kurl.pathOrUrl()) << endl;
}


void Logger::log_del(const KUrl& kurl)
{
    log_text_stream << i18nc("@item", "%1 > Deleting <filename>%2</filename>", KDateTime::currentLocalTime().toString(), kurl.pathOrUrl()) << endl;
}


void Logger::log_trash(const KUrl& kurl)
{
    log_text_stream << i18nc("@item", "%1 > Moving <filename>%2</filename> to trash", KDateTime::currentLocalTime().toString(), kurl.pathOrUrl()) << endl;
}


void Logger::log_end()
{
    log_text_stream << endl;
    for (quint8 k = 0; k < 200; ++k)
        log_text_stream << '-';
    log_text_stream << endl << endl;
}


void Logger::log_job_info(KJob*, QString info_msg, QString)
{
    log_text_stream << i18nc("@item", "%1 > %2", KDateTime::currentLocalTime().toString(), info_msg) << endl;
}


